local _G = getfenv(0);

local cfg = TipTac_Config;

local TTO_AnchorPos = {
	{ name = "Top", value = "TOP" },
	{ name = "Top Left", value = "TOPLEFT" },
	{ name = "Top Right", value = "TOPRIGHT" },
	{ name = "Bottom", value = "BOTTOM" },
	{ name = "Bottom Left", value = "BOTTOMLEFT" },
	{ name = "Bottom Right", value = "BOTTOMRIGHT" },
	{ name = "Left", value = "LEFT" },
	{ name = "Right", value = "RIGHT" },
	{ name = "Center", value = "CENTER" },
};

-- Options
local TTO_FrameCount = {};
local TTO_OptionGenres= { "General", "Scaling", "Fading", "Offsets", "Bars", "Auras", "Anchor" };
local TTO_Options = {
	-- General
	{
		{ type = "Check", var = "showTip", label = "Show TipTac Unit Tips", tip = "If you disable this option, you will only really take advantage of the anchor and scale options in TipTac" },
		{ type = "Check", var = "pvpName", label = "Use PvP Title If Available", tip = "Show PvP title for players if available" },
		{ type = "Check", var = "showTarget", label = "Show Who the Unit Targets", tip = "This will let you see who the unit targets" },
		{ type = "Check", var = "showTargetedBy", label = "Show Who Targets the Unit", tip = "When in a raid or party, it will show who from your group is targeting the unit" },
		{ type = "Slider", var = "updateFreq", label = "Tip Update Frequency", min = 0, max = 5, step = 0.05, y = -8 },
		{ type = "Check", var = "hookTips", label = "Hook Special Tips", tip = "Hook special tips to modify in appearance. Changing this options requires a UI reload to take effect.", y = -12 },
 	},
	-- Scaling
	{
		{ type = "Slider", var = "tipScale", label = "TipTacTip Scale", min = 0.4, max = 2, step = 0.05 },
		{ type = "Slider", var = "gttScale", label = "GameTooltip Scale", min = 0.4, max = 2, step = 0.05 },
	},
	-- Fading
	{
		{ type = "Slider", var = "preFadeTime", label = "Pre Fade Time", min = 0, max = 5, step = 0.05 },
		{ type = "Slider", var = "fadeTime", label = "Fade Out Time", min = 0, max = 5, step = 0.05 },
	},
	-- Offsets
	{
		{ type = "Slider", var = "mouseOffsetX", label = "Mouse Anchor X Offset", min = -200, max = 200, step = 1 },
		{ type = "Slider", var = "mouseOffsetY", label = "Mouse Anchor Y Offset", min = -200, max = 200, step = 1 },
	},
	-- Bars
	{
		{ type = "Check", var = "healthBar", label = "Show Health Bar", tip = "Will show a health bar of the unit. This supports MobHealth, and will show estimated health if installed." },
		{ type = "Check", var = "healthBarText", label = "Show Health Bar Text", tip = "Show text on health bar" },
		{ type = "Check", var = "manaBar", label = "Show Mana Bar", tip = "If the unit has mana, a mana bar will be shown.", y = -14 },
		{ type = "Check", var = "manaBarText", label = "Show Mana Bar Text", tip = "Show text on mana bar" },
		{ type = "Check", var = "powerBar", label = "Show Energy, Rage or Focus Bar", tip = "If the unit has either an energy, rage or focus bar, it will be shown.", y = -14 },
		{ type = "Check", var = "powerBarText", label = "Show Energy, Rage or Focus Bar Text", tip = "Show text on power bar" },
	},
	-- Auras
	{
		{ type = "Check", var = "showBuffs", label = "Show Unit Buffs", tip = "Show buffs of the unit" },
		{ type = "Check", var = "selfBuffsOnly", label = "Show Castable Buffs Only", tip = "This will filter out and not display any buffs you cannot cast" },
		{ type = "Check", var = "showDebuffs", label = "Show Unit Debuffs (Not Implemented Yet)", tip = "Show debuffs of the unit (not yet implemented)", y = -12 },
		{ type = "Check", var = "selfDebuffsOnly", label = "Show Removable Debuffs Only", tip = "This will filter out and not display any debuffs you cannot remove" },
		{ type = "Slider", var = "auraSize", label = "Aura Icon Dimension", min = 8, max = 40, step = 1, y = -12 },
		{ type = "Slider", var = "auraMaxRows", label = "Max Aura Rows", min = 1, max = 8, step = 1 },
		{ type = "Check", var = "showAuraCooldown", label = "Show Cooldown Models", tip = "With this option on, you will see a visual progress of the time left on the buff", y = -8 },
	},
	-- Anchor
	{
		{ type = "Check", var = "mouseAnchorUnitsOnly", label = "Only mouse anchor unit tips", tip = "When enabled, non unit tips will still be anchored to the normal anchor" },
--[[		{ type = "DropDown", var = "unitAnchorType", label = "Unit Anchor Type", list = { "Normal", "Mouse", "Smart" } },
		{ type = "DropDown", var = "unitPoint", label = "Unit Anchor Point", list = TTO_AnchorPos },

		{ type = "DropDown", var = "nonUnitAnchorType", label = "Non-Unit Anchor Type", list = { "Normal", "Mouse", "Smart" }, y = -12 },
		{ type = "DropDown", var = "nonUnitPoint", label = "Non-Unit Anchor Point", list = TTO_AnchorPos },]]
	},
};

local activePage = 1;

--------------------------------------------------------------------------------------------------------
--                                           Frame Scripts                                            --
--------------------------------------------------------------------------------------------------------
function TipTacOptions_OnLoad(self)
	tinsert(UISpecialFrames,self:GetName());
	TipTacOptionsVers:SetText("|cffffff80"..TipTacAnchor.modVers);
	if (TipTacAnchor) then
		TipTacOptionsAnchor:Show();
	end
	if (cfg.optionsLeft) and (cfg.optionsBottom) then
		self:SetPoint("BOTTOMLEFT",UIParent,"BOTTOMLEFT",cfg.optionsLeft,cfg.optionsBottom);
	end
end

-- OnShow
function TipTacOptions_OnShow(self)
	TipTacOptions_BuildPage();
	TipTacOptions_List_Update();
end

-- MouseUp
function TipTacOptions_OnMouseUp()
	this:StopMovingOrSizing();
	cfg.optionsLeft = this:GetLeft();
	cfg.optionsBottom = this:GetBottom();
end

--------------------------------------------------------------------------------------------------------
--                                            Slider Frame                                            --
--------------------------------------------------------------------------------------------------------

-- Slider Value Changed
function TipTacOptions_Slider_OnValueChanged()
	if (not TipTacOptions.updatingOptions) then
		-- Tooltip
		if (GameTooltip:IsOwned(this)) then
			GameTooltip:SetText(format("%.2f",this:GetValue()));
		end
		-- Change Edit
		_G[this:GetParent():GetName().."Edit"]:SetNumber(this:GetValue());
		-- Save & Apply
		cfg[TTO_Options[activePage][this:GetParent():GetID()].var] = this:GetValue();
		TipTac_ApplySettings();
	end
end

-- EditBox Enter Pressed
function TipTacOptions_SliderEdit_OnEnterPressed()
	_G[this:GetParent():GetName().."Slider"]:SetValue(this:GetNumber());
end

-- OnMouseWheel
function TipTacOptions_Slider_OnMouseWheel()
	this:SetValue(this:GetValue() + TTO_Options[activePage][this:GetParent():GetID()].step * arg1);
end

--------------------------------------------------------------------------------------------------------
--                                           Check Buttons                                            --
--------------------------------------------------------------------------------------------------------
function TipTacOptions_CheckBox_OnClick()
	cfg[TTO_Options[activePage][this:GetID()].var] = (this:GetChecked() == 1);
	TipTac_ApplySettings();
end

--------------------------------------------------------------------------------------------------------
--                                         Initialize Options                                         --
--------------------------------------------------------------------------------------------------------

-- Get Frame
function TipTacOptions_GetFrame(type,id,index)
	local frame = _G["TipTacOptions"..type..index];
	if (not frame) then
		TTO_FrameCount[type] = (TTO_FrameCount[type] or 0) + 1;
		if (type == "Slider") then
			frame = CreateFrame("Frame","TipTacOptionsSlider"..index,TipTacOptions,"TipTacOptionSliderTemplate");
		elseif (type == "Check") then
			frame = CreateFrame("CheckButton","TipTacOptionsCheck"..index,TipTacOptions,"TipTacOptionCheckBoxTemplate");
		end
	end
	frame:SetID(id);
	return frame;
end

-- Build Page
function TipTacOptions_BuildPage()
	local frame;
	local lastAnchor, lastType;
	local subFrame;
	local frameUseCount = {};
	-- Loop Through Options
	TipTacOptions.updatingOptions = 1;
	for index, table in ipairs(TTO_Options[activePage]) do
		-- Init the Frame
		frameUseCount[table.type] = (frameUseCount[table.type] or 0) + 1;
		frame = TipTacOptions_GetFrame(table.type,index,frameUseCount[table.type]);
		-- Anchor the Frame
		frame:ClearAllPoints();
		if (not lastAnchor) then
			frame:SetPoint("TOPLEFT",table.type == "Check" and 18 or 28,-50);
		else
			lastType = (index > 1) and (TTO_Options[activePage][index-1].type) or nil;
			if (lastType == "Slider") and (table.type == "Check") then
				frame:SetPoint("TOPLEFT",lastAnchor,"BOTTOMLEFT",-8 + (table.x or 0),-5 + (table.y or 0));
			elseif (lastType == "Check") and (table.type == "Slider") then
				frame:SetPoint("TOPLEFT",lastAnchor,"BOTTOMLEFT",8 + (table.x or 0),-5 + (table.y or 0));
			else
				frame:SetPoint("TOPLEFT",lastAnchor,"BOTTOMLEFT",(table.x or 0),(table.y or 0));
			end
		end
		lastAnchor = frame;
		-- Setup the Frame
		if (table.type == "Slider") then
			subFrame = (frame:GetName().."Slider");

			_G[subFrame]:SetMinMaxValues(table.min,table.max);
			_G[subFrame]:SetValueStep(table.step);
			_G[subFrame]:SetValue(cfg[table.var]);
			_G[frame:GetName().."Edit"]:SetNumber(cfg[table.var]);

			_G[subFrame.."Text"]:SetText(table.label);
			_G[subFrame.."Low"]:SetText(table.min);
			_G[subFrame.."High"]:SetText(table.max);
		elseif (table.type == "Check") then
			_G[frame:GetName().."Text"]:SetText(table.label);
			frame.tooltipText = table.tip;
			frame:SetChecked(cfg[table.var]);
		end
		-- Show
		frame:Show();
	end
	-- Hide Unused Frames
	for type, count in pairs(TTO_FrameCount) do
		for i = (frameUseCount[type] or 0) + 1, count do
			_G["TipTacOptions"..type..i]:Hide();
		end
	end
	-- Reset
	TipTacOptions.updatingOptions = nil;
end

--------------------------------------------------------------------------------------------------------
--                                             Page List                                              --
--------------------------------------------------------------------------------------------------------

function TipTacOptions_List_Update()
	TipTacOptionsList:SetHeight(18 * #TTO_OptionGenres + 36);
	local button;
	for index, name in ipairs(TTO_OptionGenres) do
		button = _G["TipTacOptionsListEntry"..index] or CreateFrame("Button","TipTacOptionsListEntry"..index,TipTacOptionsList,"TipTacOptionsListEntryTemplate");
		_G["TipTacOptionsListEntry"..index.."Text"]:SetText((index == activePage and "|cffffffff"  or "")..name);
		button.page = index;
		button:ClearAllPoints();
		if (index == 1) then
			button:SetPoint("TOPLEFT",TipTacOptionsListOutline,"TOPLEFT",5,-6);
		else
			button:SetPoint("TOPLEFT","TipTacOptionsListEntry"..(index - 1),"BOTTOMLEFT");
		end
	end
end

function TipTacOptions_List_OnClick(self,button)
	activePage = self.page;
	PlaySound("igMainMenuOptionCheckBoxOn");
	TipTacOptions_BuildPage();
	TipTacOptions_List_Update();
end

--------------------------------------------------------------------------------------------------------
--                                                Misc                                                --
--------------------------------------------------------------------------------------------------------
function TipTacOptions_Anchor()
	if (TipTacAnchor:IsVisible()) then
		TipTacAnchor:Hide();
	else
		TipTacAnchor:Show();
	end
end